@@ -14,8 +14,9 @@ import sys
 import six
 import numpy as np
 from patsy import PatsyError
-from patsy.compat import triu_indices, tril_indices, diag_indices
-from patsy.util import repr_pretty_delegate, repr_pretty_impl
+from patsy.util import (repr_pretty_delegate, repr_pretty_impl,
+                        safe_issubdtype,
+                        no_pickling, assert_no_pickling)
 
 class ContrastMatrix(object):
     """A simple container for a matrix used for coding categorical factors.
@@ -46,6 +47,8 @@ class ContrastMatrix(object):
     def _repr_pretty_(self, p, cycle):
         repr_pretty_impl(p, self, [self.matrix, self.column_suffixes])
 
+    __getstate__ = no_pickling
+
 def test_ContrastMatrix():
     cm = ContrastMatrix([[1, 0], [0, 1]], ["a", "b"])
     assert np.array_equal(cm.matrix, np.eye(2))
@@ -56,6 +59,8 @@ def test_ContrastMatrix():
     from nose.tools import assert_raises
     assert_raises(PatsyError, ContrastMatrix, [[1], [0]], ["a", "b"])
 
+    assert_no_pickling(cm)
+
 # This always produces an object of the type that Python calls 'str' (whether
 # that be a Python 2 string-of-bytes or a Python 3 string-of-unicode). It does
 # *not* make any particular guarantees about being reversible or having other
@@ -108,7 +113,7 @@ def _dummy_code(levels):
 def _get_level(levels, level_ref):
     if level_ref in levels:
         return levels.index(level_ref)
-    if isinstance(level_ref, int):
+    if isinstance(level_ref, six.integer_types):
         if level_ref < 0:
             level_ref += len(levels)
         if not (0 <= level_ref < len(levels)):
@@ -128,6 +133,12 @@ def test__get_level():
     assert_raises(PatsyError, _get_level, ["a", "b"], -3)
     assert_raises(PatsyError, _get_level, ["a", "b"], "c")
 
+    if not six.PY3:
+        assert _get_level(["a", "b", "c"], long(0)) == 0
+        assert _get_level(["a", "b", "c"], long(-1)) == 2
+        assert _get_level([2, 1, 0], long(0)) == 2
+
+
 class Treatment(object):
     """Treatment coding (also known as dummy coding).
 
@@ -176,6 +187,8 @@ class Treatment(object):
         names = _name_levels("T.", levels[:reference] + levels[reference + 1:])
         return ContrastMatrix(contrasts, names)
 
+    __getstate__ = no_pickling
+
 def test_Treatment():
     t1 = Treatment()
     matrix = t1.code_with_intercept(["a", "b", "c"])
@@ -272,6 +285,8 @@ class Poly(object):
     def code_without_intercept(self, levels):
         return self._code_either(False, levels)
 
+    __getstate__ = no_pickling
+
 def test_Poly():
     t1 = Poly()
     matrix = t1.code_with_intercept(["a", "b", "c"])
@@ -311,6 +326,11 @@ def test_Poly():
                         [1, 0.293294230042706, -0.762000762001143],
                         [1, 0.513264902574736, 0.635000635000952]])
 
+    from nose.tools import assert_raises
+    assert_raises(PatsyError,
+                  Poly(scores=[0, 1]).code_with_intercept,
+                  ["a", "b", "c"])
+
     matrix = t1.code_with_intercept(list(range(6)))
     assert matrix.column_suffixes == [".Constant", ".Linear", ".Quadratic",
                                       ".Cubic", "^4", "^5"]
@@ -378,6 +398,8 @@ class Sum(object):
         included_levels = levels[:omit_i] + levels[omit_i + 1:]
         return ContrastMatrix(matrix, _name_levels("S.", included_levels))
 
+    __getstate__ = no_pickling
+
 def test_Sum():
     t1 = Sum()
     matrix = t1.code_with_intercept(["a", "b", "c"])
@@ -456,8 +478,8 @@ class Helmert(object):
 
         #r-like
         contr = np.zeros((n, n - 1))
-        contr[1:][diag_indices(n - 1)] = np.arange(1, n)
-        contr[triu_indices(n - 1)] = -1
+        contr[1:][np.diag_indices(n - 1)] = np.arange(1, n)
+        contr[np.triu_indices(n - 1)] = -1
         return contr
 
     def code_with_intercept(self, levels):
@@ -471,6 +493,8 @@ class Helmert(object):
         return ContrastMatrix(contrast,
                               _name_levels("H.", levels[1:]))
 
+    __getstate__ = no_pickling
+
 def test_Helmert():
     t1 = Helmert()
     for levels in (["a", "b", "c", "d"], ("a", "b", "c", "d")):
@@ -514,14 +538,14 @@ class Diff(object):
         contr = np.zeros((nlevels, nlevels-1))
         int_range = np.arange(1, nlevels)
         upper_int = np.repeat(int_range, int_range)
-        row_i, col_i = triu_indices(nlevels-1)
+        row_i, col_i = np.triu_indices(nlevels-1)
         # we want to iterate down the columns not across the rows
         # it would be nice if the index functions had a row/col order arg
         col_order = np.argsort(col_i)
         contr[row_i[col_order],
               col_i[col_order]] = (upper_int-nlevels)/float(nlevels)
         lower_int = np.repeat(int_range, int_range[::-1])
-        row_i, col_i = tril_indices(nlevels-1)
+        row_i, col_i = np.tril_indices(nlevels-1)
         # we want to iterate down the columns not across the rows
         col_order = np.argsort(col_i)
         contr[row_i[col_order]+1, col_i[col_order]] = lower_int/float(nlevels)
@@ -536,6 +560,8 @@ class Diff(object):
         contrast = self._diff_contrast(levels)
         return ContrastMatrix(contrast, _name_levels("D.", levels[:-1]))
 
+    __getstate__ = no_pickling
+
 def test_diff():
     t1 = Diff()
     matrix = t1.code_with_intercept(["a", "b", "c", "d"])
@@ -567,7 +593,7 @@ def code_contrast_matrix(intercept, levels, contrast, default=None):
     if isinstance(contrast, ContrastMatrix):
         return contrast
     as_array = np.asarray(contrast)
-    if np.issubdtype(as_array.dtype, np.number):
+    if safe_issubdtype(as_array.dtype, np.number):
         return ContrastMatrix(as_array,
                               _name_levels("custom", range(as_array.shape[1])))
     if intercept:
